// fuzzydefense.cpp : Defines the entry point for the console application.
//
// This is the file that contains main(). It registers all the necessary
// GLUT functions and keeps track of the animation objects. It also
// advances the simulation and does the actual drawing of the objects

#include <stdio.h>
#include <math.h>
#include <iostream>
#include "generic_object.h"
#include "simulation.h"

#define GL_PI 3.1459f

// Keep track of windows changing width and height
GLfloat windowWidth;
GLfloat windowHeight;

// these are used for on-screen picking
GLfloat realWidth;
GLfloat realHeight;

// in milliseconds, how many times the simulation is updated
unsigned int stepSize = 33;
GLfloat pl_pos[2];		// position of player 1
GLfloat pl2_pos[2];		// position of player 2
GLfloat pu_pos[2];		// position of the puck
GLfloat tm_pos[2];		// position of the teammate
GLfloat opp_pos[2];		// position of the opponent

unsigned short int picker;
bool grouped = false;
GLfloat temp_pos[2];

short int possession = 0;	// used indicate who has the puck
							// possession = 1, player's team
							// possession = -1, opponent's team
							// possession = 0, puck is loose

unsigned short int player_score = 0;
unsigned short int opponent_score = 0;
signed short int game_score;	// game_score = player_score-opponent_score;
unsigned short int game_time = 0;	// game time in minutes (0-60)

GLfloat init_puck_pos[2] = {110.0f, 50.0f};	// center ice
GLfloat puck_col[3] = {0.0f, 0.0f, 0.0f};	// black
GLfloat puck_dot_size = 5.0f;				// puck size

GLfloat init_opp_pos[2] = {114.0f, 50.0f};	// in faceoff circle
GLfloat opp_col[3] = {1.0f, 1.0f, 0.0f};	// yellow
GLfloat opp_dot_size = 8.0f;				// big and tuff!

GLfloat init_pl_pos[2] = {85.0f, 19.0f};	// right side
GLfloat pl_col[3] = {0.68f, 1.0f, 0.18f};	// green!
GLfloat pl_dot_size = 8.0f;

GLfloat init_pl2_pos[2] = {85.0f, 81.0f};	// left side

GLfloat init_tm_pos[2] = {106.0, 50.0f};	// in faceoff circle
GLfloat tm_col[3] = {0.68f, 1.0f, 0.18f};	// green!
GLfloat tm_dot_size = 8.0f;

// make the puck
generic_object puck(init_puck_pos, puck_col, puck_dot_size);

// make an opponent
generic_object opponent(init_opp_pos, opp_col, opp_dot_size);

// make a defenseman
generic_object player(init_pl_pos, pl_col, pl_dot_size);

// make another defenseman
generic_object player2(init_pl2_pos, pl_col, pl_dot_size);

// make a teammate
generic_object teammate(init_tm_pos, tm_col, tm_dot_size);

// make a simulation object
simulation sim(stepSize);

using namespace std;

// Function that draws the rink surface - all dots and lines!
void drawRink()
{
	GLfloat ax1 = 5.0f;
	GLfloat ay1 = 5.0f;
	GLsizei rsize = 50;
	GLfloat r_width = 100.0f;
	GLfloat r_length = 220.0f;

	GLfloat neutralDotSize = 4.0f;
	GLfloat mainFaceOffDotSize = 4.0f;
	GLfloat faceOffDotSize = 2.0f;
	GLfloat cornerDotSize = 1.5f;
	GLfloat angle, x, y;

  	// Set current drawing color
	//		   R	 G	   B
	glColor3f(1.0f, 1.0f, 1.0f);

	// Draw the ice surface
	glRectf(ax1, ay1, ax1+r_length, ay1+r_width);


	// Left end line
	glColor3f(0.0f, 0.0f, 0.0f);
	glBegin(GL_LINE_STRIP);
		glVertex3f(10.0f, 20.5f, 0.0f);
		glVertex3f(10.0f, 79.5f, 0.0f);
	glEnd();

	// Left goal line
	glColor3f(1.0f, 0.0f, 0.0f);
	glBegin(GL_LINE_STRIP);
		glVertex3f(23.0f, 7.5f, 0.0f);
		glVertex3f(23.0f, 92.5f, 0.0f);
	glEnd();

	// Left blue line
	glColor3f(0.0f, 0.0f, 1.0f);
	glBegin(GL_LINE_STRIP);
		glVertex3f(83.0f, 7.5f, 0.0f);
		glVertex3f(83.0f, 92.5f, 0.0f);
	glEnd();

	// Red line
	glColor3f(1.0f, 0.0f, 0.0f);
	glBegin(GL_LINE_STRIP);
		glVertex3f(110.0f, 7.5f, 0.0f);
		glVertex3f(110.0f, 92.5f, 0.0f);
	glEnd();

	// Right blue line
	glColor3f(0.0f, 0.0f, 1.0f);
	glBegin(GL_LINE_STRIP);
		glVertex3f(137.0f, 7.5f, 0.0f);
		glVertex3f(137.0f, 92.5f, 0.0f);
	glEnd();

	// Right goal line
	glColor3f(1.0f, 0.0f, 0.0f);
	glBegin(GL_LINE_STRIP);
		glVertex3f(197.0f, 7.5f, 0.0f);
		glVertex3f(197.0f, 92.5f, 0.0f);
	glEnd();

	// Right end line
	glColor3f(0.0f, 0.0f, 0.0f);
	glBegin(GL_LINE_STRIP);
		glVertex3f(210.0f, 20.5f, 0.0f);
		glVertex3f(210.0f, 79.5f, 0.0f);
	glEnd();

	// Bottom end board
	glColor3f(0.0f, 0.0f, 0.0f);
	glBegin(GL_LINE_STRIP);
		glVertex3f(23.0f, 7.5f, 0.0f);
		glVertex3f(197.0f, 7.5f, 0.0f);
	glEnd();

	// Top end board
	glColor3f(0.0f, 0.0f, 0.0f);
	glBegin(GL_LINE_STRIP);
		glVertex3f(23.0f, 92.5f, 0.0f);
		glVertex3f(197.0f, 92.5f, 0.0f);
	glEnd();

	// Left goal
	glColor3f(0.0f, 0.0f, 0.0f);
	glBegin(GL_LINE_STRIP);
		glVertex3f(23.0f, 47.0f, 0.0f);
		glVertex3f(20.0f, 47.0f, 0.0f);
		glVertex3f(20.0f, 53.0f, 0.0f);
		glVertex3f(23.0f, 53.0f, 0.0f);
	glEnd();

	// Right goal
	glColor3f(0.0f, 0.0f, 0.0f);
	glBegin(GL_LINE_STRIP);
		glVertex3f(197.0f, 47.0f, 0.0f);
		glVertex3f(200.0f, 47.0f, 0.0f);
		glVertex3f(200.0f, 53.0f, 0.0f);
		glVertex3f(197.0f, 53.0f, 0.0f);
	glEnd();



	// Bottom left corner
	glPointSize(cornerDotSize);
	for (angle = GL_PI; angle <= 1.5f*GL_PI; angle += 0.05f)
	{
		x = 13.0f*sin(angle)+23.0f;
		y = 13.0f*cos(angle)+20.5f;

		// draw the circle
		glBegin(GL_POINTS);
			glVertex3f(x, y, 0.0f);
		glEnd();
	}

	// Top left corner
	glPointSize(cornerDotSize);
	for (angle = 1.5f*GL_PI; angle <= 2.0f*GL_PI; angle += 0.05f)
	{
		x = 13.0f*sin(angle)+23.0f;
		y = 13.0f*cos(angle)+79.5f;

		// draw the circle
		glBegin(GL_POINTS);
			glVertex3f(x, y, 0.0f);
		glEnd();
	}

	// Top right corner
	glPointSize(cornerDotSize);
	for (angle = 0.0f; angle <= GL_PI/2.0f; angle += 0.05f)
	{
		x = 13.0f*sin(angle)+197.0f;
		y = 13.0f*cos(angle)+79.5f;

		// draw the circle
		glBegin(GL_POINTS);
			glVertex3f(x, y, 0.0f);
		glEnd();
	}

	// Bottom right corner
	glPointSize(cornerDotSize);
	for (angle = GL_PI/2.0f; angle <= GL_PI; angle += 0.05f)
	{
		x = 13.0f*sin(angle)+197.0f;
		y = 13.0f*cos(angle)+20.5f;

		// draw the circle
		glBegin(GL_POINTS);
			glVertex3f(x, y, 0.0f);
		glEnd();
	}

	// Bottom left neutral dot
	glColor3f(1.0f, 0.0f, 0.0f);
	glPointSize(neutralDotSize);
	glBegin(GL_POINTS);
		glVertex3f(88.0f, 28.0f, 0.0f);
	glEnd();

	// Top left neutral dot
	glColor3f(1.0f, 0.0f, 0.0f);
	glPointSize(neutralDotSize);
	glBegin(GL_POINTS);
		glVertex3f(88.0f, 72.0f, 0.0f);
	glEnd();

	// Bottom right neutral dot
	glColor3f(1.0f, 0.0f, 0.0f);
	glPointSize(neutralDotSize);
	glBegin(GL_POINTS);
		glVertex3f(132.0f, 28.0f, 0.0f);
	glEnd();

	// Top right neutral dot
	glColor3f(1.0f, 0.0f, 0.0f);
	glPointSize(neutralDotSize);
	glBegin(GL_POINTS);
		glVertex3f(132.0f, 72.0f, 0.0f);
	glEnd();

	// Main faceoff circle
	glPointSize(mainFaceOffDotSize);
	glBegin(GL_POINTS);
			glVertex3f(110.0, 50.0, 0.0f);
	glEnd();
	for (angle = 0.0f; angle <= 2.0f*GL_PI; angle += 0.05f)
	{
		x = 15.0f*sin(angle)+110.0f;
		y = 15.0f*cos(angle)+50.0f;

		// draw the circle
		glBegin(GL_POINTS);
			glVertex3f(x, y, 0.0f);
		glEnd();
	}

	// Bottom left faceoff circle
	glPointSize(mainFaceOffDotSize);
	glBegin(GL_POINTS);
			glVertex3f(43.0, 28.0, 0.0f);
	glEnd();
	glPointSize(faceOffDotSize);
	for (angle = 0.0f; angle <= 2.0f*GL_PI; angle += 0.05f)
	{
		x = 7.5f*sin(angle)+43.0f;
		y = 7.5f*cos(angle)+28.0f;

		// draw the circle
		glBegin(GL_POINTS);
			glVertex3f(x, y, 0.0f);
		glEnd();
	}

	// Top left faceoff circle
	glPointSize(mainFaceOffDotSize);
	glBegin(GL_POINTS);
		glVertex3f(43.0, 72.0, 0.0f);
	glEnd();
	glPointSize(faceOffDotSize);
	for (angle = 0.0f; angle <= 2.0f*GL_PI; angle += 0.05f)
	{
		x = 7.5f*sin(angle)+43.0f;
		y = 7.5f*cos(angle)+72.0f;

		// draw the circle
		glBegin(GL_POINTS);
			glVertex3f(x, y, 0.0f);
		glEnd();
	}

	// Top right faceoff circle
	glPointSize(mainFaceOffDotSize);
	glBegin(GL_POINTS);
		glVertex3f(177.0, 72.0, 0.0f);
	glEnd();
	glPointSize(faceOffDotSize);
	for (angle = 0.0f; angle <= 2.0f*GL_PI; angle += 0.05f)
	{
		x = 7.5f*sin(angle)+177.0f;
		y = 7.5f*cos(angle)+72.0f;

		// draw the circle
		glBegin(GL_POINTS);
			glVertex3f(x, y, 0.0f);
		glEnd();
	}

	// Bottom right faceoff circle
	glPointSize(mainFaceOffDotSize);
	glBegin(GL_POINTS);
		glVertex3f(177.0, 28.0, 0.0f);
	glEnd();
	glPointSize(faceOffDotSize);
	for (angle = 0.0f; angle <= 2.0f*GL_PI; angle += 0.05f)
	{
		x = 7.5f*sin(angle)+177.0f;
		y = 7.5f*cos(angle)+28.0f;

		// draw the circle
		glBegin(GL_POINTS);
			glVertex3f(x, y, 0.0f);
		glEnd();
	}
}

// Function that draws the possession indicator
// Draws a box around the colour of the team with the puck
void drawPossessionIndicator(void)
{
	// Player team indicator
	glColor3f(pl_col[0], pl_col[1], pl_col[2]);
	glPointSize(pl_dot_size);
	glBegin(GL_POINTS);
		glVertex3f(218.0f, 60.0f, 0.0f);
	glEnd();

	// Opponent team indicator
	glColor3f(opp_col[0], opp_col[1], opp_col[2]);
	glPointSize(opp_dot_size);
	glBegin(GL_POINTS);
		glVertex3f(218.0f, 40.0f, 0.0f);
	glEnd();

	if (possession == 1) // player's team has puck
	{
		glColor3f(0.0f, 0.0f, 0.0f);
		glBegin(GL_LINE_STRIP);
			glVertex3f(215.0f, 57.0f, 0.0f);
			glVertex3f(215.0f, 63.0f, 0.0f);
			glVertex3f(221.0f, 63.0f, 0.0f);
			glVertex3f(221.0f, 57.0f, 0.0f);
			glVertex3f(215.0f, 57.0f, 0.0f);
		glEnd();
	}

	else if (possession == -1) // opponent has puck
	{
		// Bottom end board
		glColor3f(0.0f, 0.0f, 0.0f);
		glBegin(GL_LINE_STRIP);
			glVertex3f(215.0f, 37.0f, 0.0f);
			glVertex3f(215.0f, 43.0f, 0.0f);
			glVertex3f(221.0f, 43.0f, 0.0f);
			glVertex3f(221.0f, 37.0f, 0.0f);
			glVertex3f(215.0f, 37.0f, 0.0f);
		glEnd();
	}
}


// Function that renders the scene
void RenderScene(void)
	{
	// Clear the window with current clearing color
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glTranslatef(0.0f, 0.0f, -1.0f);

	// draw rink
	drawRink();

	// draw the possession indicator
	drawPossessionIndicator();

	// draw puck
	puck.draw();

	// draw player
	player.draw();

	// draw 2nd player
	player2.draw();

	// draw teammate
	teammate.draw();

	// draw opponent
	opponent.draw();

	// Flush drawing commands
    glutSwapBuffers();
	}

// Setup the rendering state
void SetupRC(void)
    {
    // Set clear color
    glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
    }

// Called by GLUT library when the window has chanaged size
void ChangeSize(GLsizei w, GLsizei h)
	{
	// Prevent a divide by zero
	if(h == 0)
		h = 1;

	// Set Viewport to window dimensions
    glViewport(0, 0, w, h);

	// Reset coordinate system
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// we need these for on-screen picking
	realWidth = w;
	realHeight = h;

	// Keep the square square, this time, save calculated
	// width and height for later use
	if (w <= h) 
		{
		windowHeight = 110.0f*h/w;
		windowWidth = 110.0f;
		}
    else 
		{
		windowWidth = 110.0f*w/h;
		windowHeight = 110.0f;
		}

	// Set the clipping volume
	glOrtho(0.0f, windowWidth, 0.0f, windowHeight, 1.0f, -1.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	}

// Function that is called by GLUT for every tick of the simulation
// It calls the simulation object to update the positions of the objects
// then forces a re-draw of the scene
void animate(int value)
{
	// get the various game parameters
	player.getPosition(pl_pos);
	player2.getPosition(pl2_pos);
	puck.getPosition(pu_pos);
	teammate.getPosition(tm_pos);
	opponent.getPosition(opp_pos);
	game_score = player_score - opponent_score;

	if (game_score > 1)
	{
		game_score = 1;
	}
	else if (game_score < -1)
	{
		game_score = -1;
	}

	// calls the simulation object to update the object positions
	sim.nextStep(pl_pos, pl2_pos, pu_pos, tm_pos, opp_pos, possession, game_score, game_time);

	// set the positions
	player.setPosition(pl_pos);
	player2.setPosition(pl2_pos);

	// force a re-draw
	glutPostRedisplay();

	// ensures the next tick of the simulation happens again
	glutTimerFunc(stepSize, animate, 1);
}

// Function that handles any keyboard input to the sim
void keyBoardHandler(unsigned char key, int x, int y)
{
	switch(key){
	case 'q':          // quit
		{
			exit(0);
			break;
		}

	case 'g':          // quit
		{
			if (grouped == true)
			{
				grouped = false;
				cout << "Group selection disabled\n";
			}
			else
			{
				grouped = true;
				cout << "Group selection enabled\n";
			}
			break;
		}

		case 's':	// change the score
		{
			bool halt = false;
			short int temp;

			cout << "The current score is...\n";
			cout << "Player = " << player_score << "\n";
			cout << "Opponent = " << game_score << "\n";
			while (halt == false)
			{
				cout << "Please enter in score for Player\n";
				cin >> temp;
				if (temp < 0)
				{
					cout << "You cannot have a negative score, please try again...\n";
				}
				else
				{
					player_score = temp;
					halt = true;
				}
			}
			halt = false;
			while (halt == false)
			{
				cout << "Please enter in score for Opponent\n";
				cin >> temp;
				if (temp < 0)
				{
					cout << "You cannot have a negative score, please try again...\n";
				}
				else
				{
					opponent_score = temp;
					halt = true;
				}
			}

			cout << "The score is now...\n";
			cout << "Player = " << player_score << "\n";
			cout << "Opponent = " << opponent_score << "\n";
			cin.ignore(1, '\n');
			break;
		}

		case 't':		// change the game time
		{
			bool halt = false;
			short int period;

			if ((game_time >=0) || (game_time < 20))
			{
				cout << "We are in the 1st period, with " << game_time << " minutes played...\n";
			}

			else if ((game_time >=20) || (game_time < 40))
			{
				cout << "We are in the 2nd period, with " << game_time-20 << " minutes played...\n";
			}

			else if ((game_time >=40) || (game_time < 60))
			{
				cout << "We are in the 3rd period, with " << game_time-40 << " minutes played...\n";
			}

			while (halt == false)
			{
				cout << "What period do you want to set the game to?\n";
				cin >> period;
				if ((period != 1) && (period != 2) && (period != 3))
				{
					cout << "Invalid entry, please try again...\n";
				}
				else
				{
					halt = true;
				}
			}
			halt = false;
			while (halt == false)
			{
				cout << "Please enter in the amount of time expired in that period\n";
				cin >> game_time;
				if ((game_time < 0) || (game_time > 20))
				{
					cout << "Invalid entry, please try again...\n";
				}
				else
				{
					cout << "We are now in period " << period << ", with " << game_time << " minutes played...\n";
					if (period == 2)
					{
						game_time = game_time + 20;
					}
					if (period == 3)
					{
						game_time = game_time + 40;
					}
					halt = true;
				}
			}
			cin.ignore(1, '\n');
			break;
		}

		default:
		{
			break;
		}
	} // end switch
	
	// not necessary, but eh, why not?
	glutPostRedisplay();
}

// Function that handles mouse button clicks
void mouseHandler(int button, int state, int x, int y)
{
	GLfloat pos[2];
	int diffX, diffY;

	// translates window coordinates to drawing coordinates
	x = (int)(windowWidth*x/realWidth);
	y = (int)(windowHeight*(realHeight-y)/realHeight);

	player.getPosition(pos);
	diffX = abs((int)pos[0]-x);
	diffY = abs((int)pos[1]-y);

	// check if the first defenseman is selected
	if ((button == GLUT_LEFT_BUTTON) && (state == GLUT_DOWN) && (diffX < 2) && (diffY < 2)) 
	{
		picker = 1;
		return;
	}

	puck.getPosition(pos);
	diffX = abs((int)pos[0]-x);
	diffY = abs((int)pos[1]-y);

	// check if the puck is selected
	if ((button == GLUT_LEFT_BUTTON) && (state == GLUT_DOWN) && (diffX < 2) && (diffY < 2)) 
	{
		picker = 2;
		return;
	}

	opponent.getPosition(pos);
	diffX = abs((int)pos[0]-x);
	diffY = abs((int)pos[1]-y);

	// check if the opponent is selected
	if ((button == GLUT_LEFT_BUTTON) && (state == GLUT_DOWN) && (diffX < 2) && (diffY < 2)) 
	{
		picker = 3;
		return;
	}

	teammate.getPosition(pos);
	diffX = abs((int)pos[0]-x);
	diffY = abs((int)pos[1]-y);

	// check if the teammate is selected
	if ((button == GLUT_LEFT_BUTTON) && (state == GLUT_DOWN) && (diffX < 2) && (diffY < 2)) 
	{
		picker = 4;
		return;
	}

	player2.getPosition(pos);
	diffX = abs((int)pos[0]-x);
	diffY = abs((int)pos[1]-y);

	// check if the second defenseman is selected
	if ((button == GLUT_LEFT_BUTTON) && (state == GLUT_DOWN) && (diffX < 2) && (diffY < 2)) 
	{
		picker = 5;
		return;
	}

	// reset the picker, not explicitly necessary
	if ((button == GLUT_LEFT_BUTTON) && (state == GLUT_UP))
	{
		picker = 0;
	}
}

// Function that handles when the mouse is moving in the window
// and the button is down
void motionHandler(int x, int y)
{
		// translates window coords into drawing coords
		x = (int)(windowWidth*x/realWidth);
		y = (int)(windowHeight*(realHeight-y)/realHeight);

		if (y > 92) // prevents dragging to far up
		{
			y = 92;
		}
		if (y < 8)	// prevents dragging to far down
		{
			y = 8;
		}
		if (x > 210) // prevents dragging to far right
		{
			x = 210;
		}
		if (x < 10)	// prevents dragging to far left
		{
			x = 10;
		}
		temp_pos[0] = (GLfloat)x;
		temp_pos[1] = (GLfloat)y;

	if (picker == 1)
	{
		player.setPosition(temp_pos);	
	}
	else if (picker == 2)
	{
		puck.setPosition(temp_pos);
	}
	else if (picker == 3)
	{
		opponent.setPosition(temp_pos);
		if ((grouped == true) && (possession == -1))
		{
			temp_pos[0] = temp_pos[0] - 1.0f;
			puck.setPosition(temp_pos);
		}
	}
	else if (picker == 4)
	{
		teammate.setPosition(temp_pos);
		if ((grouped == true) && (possession == 1))
		{
			temp_pos[0] = temp_pos[0] + 1.0f;
			puck.setPosition(temp_pos);
		}
	}
	else if (picker == 5)
	{
		player2.setPosition(temp_pos);
	}

	glutPostRedisplay();
}

int main(int argc, char* argv[])
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
	glutInitWindowSize(730, 350);
	glutCreateWindow("Fuzzy Defense");
	glutDisplayFunc(RenderScene);
    glutReshapeFunc(ChangeSize);
	glutTimerFunc(stepSize, animate, 1);
	glutKeyboardFunc(keyBoardHandler);
	glutMouseFunc(mouseHandler);
	glutMotionFunc(motionHandler);

	// set the highlight colour for first defenseman
	GLfloat mColour[3] = {0.0f, 0.0f, 0.0f};
	player.setMarkerColour(mColour);

	mColour[0] = 1.0f;
	mColour[1] = 0.0f;
	mColour[2] = 0.0f;

	// set the highlight colour for second defenseman
	player2.setMarkerColour(mColour);

	SetupRC();

	glutMainLoop();

	return 0;
}

