#include <iostream>
#include "simulation.h"


using namespace std;

#define PUCK_POSITION 0
#define SEPARATION 0
#define POSSESSION 1
#define GAME_SCORE 2
#define GAME_TIME 3

// Constructor
simulation::simulation(int sSize)
{
	stepSize = (float)sSize;

	// create and load the models
	fuzzy_position = ffll_new_model();
	fuzzy_position2 = ffll_new_model();
	fuzzy_velocity = ffll_new_model();

    int ret_val = (int)ffll_load_fcl_file(fuzzy_position, "position_timid.fcl");
	if (ret_val < 0)
	{
		cout << "Error Opening fcl file";
		// make sure the "working directory" in "Project | Settings..."
		// is set to the executable's directory if running from the MSVC IDE
	}

	ret_val = (int)ffll_load_fcl_file(fuzzy_position2, "position_aggr.fcl");
	if (ret_val < 0)
	{
		cout << "Error Opening fcl file";
		// make sure the "working directory" in "Project | Settings..."
		// is set to the executable's directory if running from the MSVC IDE
	}

	ret_val = (int)ffll_load_fcl_file(fuzzy_velocity, "player_velocity.fcl");
	if (ret_val < 0)
	{
		cout << "Error Opening fcl file";
		// make sure the "working directory" in "Project | Settings..."
		// is set to the executable's directory if running from the MSVC IDE
	}

	// create a child for each model
	// a model can have many children, but we only need one in our case
    position_child = ffll_new_child(fuzzy_position);
	velocity_child = ffll_new_child(fuzzy_velocity);
	position2_child = ffll_new_child(fuzzy_position2);
}

// Method that calculates the new positions and velocities for each of the defenseman
void simulation::nextStep(GLfloat *player_pos, GLfloat *player2_pos, GLfloat *puck_pos, GLfloat *tm_pos, GLfloat *opp_pos, short int &possession, short int &game_score, unsigned short int &game_time)
{
	float separation;
	float velocity;
	float delta;
	float desired_pos;
	int oX, oY, tX, tY;

	// feed in inputs to the fuzzy engine for position, first defenseman
	ffll_set_value(fuzzy_position, position_child, PUCK_POSITION, puck_pos[0]-110.0);
	ffll_set_value(fuzzy_position, position_child, POSSESSION, possession);
	ffll_set_value(fuzzy_position, position_child, GAME_SCORE, game_score);
	ffll_set_value(fuzzy_position, position_child, GAME_TIME, game_time);

	// feed in inputs to the fuzzy engine for position, second defenseman
	ffll_set_value(fuzzy_position2, position2_child, PUCK_POSITION, puck_pos[0]-110.0);
	ffll_set_value(fuzzy_position2, position2_child, POSSESSION, possession);
	ffll_set_value(fuzzy_position2, position2_child, GAME_SCORE, game_score);
	ffll_set_value(fuzzy_position2, position2_child, GAME_TIME, game_time);

	// get the desired position for the first defenseman
	desired_pos = ffll_get_output_value(fuzzy_position, position_child);

	// what is the separation between current and desired positions
	separation = desired_pos-player_pos[0]+110.0;

	// calculate new velocity based on separation value
	ffll_set_value(fuzzy_velocity, velocity_child, SEPARATION, separation);
	velocity = ffll_get_output_value(fuzzy_velocity, velocity_child);

	// calculate delta in X position based on velocity
	delta = velocity*(stepSize/1000.0);

	// change x coord for d-man 1
	player_pos[0] = player_pos[0] + delta;

	// dead simple way to animate d-man in y direction
	// I know it's crude, but I have a deadline to meet
	if (abs(puck_pos[1]-player_pos[1]) > 2)
	{
		if (puck_pos[1]-player_pos[1] > 0)
		{
			velocity = 15.0;
		}
		else
		{
			velocity = -15.0;
		}
		delta = velocity*(stepSize/1000.0);
	}
	else
	{
		delta = 0.0;
	}

	// these conditionals keep the d-man in his skating lane
	// a good d-man always knows his territory!
	if (player_pos[1]+delta > 29.0)
	{
		player_pos[1] = 29.0f;
	}
	else if (player_pos[1]+delta < 9.0)
	{
		player_pos[1] = 9.0f;
	}
	else // update his y position
	{
		player_pos[1] = player_pos[1] + delta;
	}

	// now for player 2, same thing as above
	desired_pos = ffll_get_output_value(fuzzy_position2, position2_child);

	separation = desired_pos-player2_pos[0]+110.0;

	ffll_set_value(fuzzy_velocity, velocity_child, SEPARATION, separation);
	velocity = ffll_get_output_value(fuzzy_velocity, velocity_child);

	delta = velocity*(stepSize/1000.0);

	player2_pos[0] = player2_pos[0] + delta;

	if (abs(puck_pos[1]-player2_pos[1]) > 2)
	{
		if (puck_pos[1]-player2_pos[1] > 0)
		{
			velocity = 15.0;
		}
		else
		{
			velocity = -15.0;
		}
		delta = velocity*(stepSize/1000.0);
	}
	else
	{
		delta = 0.0;
	}

	if (player2_pos[1]+delta > 91.0)
	{
		player2_pos[1] = 91.0f;
	}
	else if (player2_pos[1]+delta < 71.0)
	{
		player2_pos[1] = 71.0f;
	}
	else
	{
		player2_pos[1] = player2_pos[1] + delta;
	}

	// check who has puck
	// the way I wrote this, the opponent always
	// wins possess in a scrum, feel free to change it
	oX = abs((int)opp_pos[0]-puck_pos[0]);
	oY = abs((int)opp_pos[1]-puck_pos[1]);

	tX = abs((int)tm_pos[0]-puck_pos[0]);
	tY = abs((int)tm_pos[1]-puck_pos[1]);

	if ((oX < 2) && (oY < 2))
	{
		possession = -1; // opponent gets puck
	}
	else if ((tX < 2) && (tY < 2))
	{
		possession = 1;	// player gets puck
	}
	else
	{
		possession = 0; // loose puck!
	}
}
